from __future__ import annotations

from avilla.core.context import Context
from avilla.core.event import RelationshipCreated, RelationshipDestroyed
from avilla.core.selector import Selector
from avilla.qqapi.capability import QQAPICapability
from avilla.qqapi.collector.connection import ConnectionCollector


class QQAPIEventRelationshipPerform((m := ConnectionCollector())._):
    m.namespace = "avilla.protocol/qqapi::event"
    m.identify = "relationship"

    @m.entity(QQAPICapability.event_callback, event_type="guild_create")
    async def guild_create(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["id"]))
        inviter = guild.member(str(raw_event["op_user_id"]))
        context = Context(
            account,
            inviter,
            guild.member(account_route["account"]),
            guild,
            guild.member(account_route["account"]),
        )
        return RelationshipCreated(context)

    @m.entity(QQAPICapability.event_callback, event_type="guild_delete")
    async def guild_delete(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["id"]))
        operator = guild.member(str(raw_event["op_user_id"]))
        context = Context(
            account,
            operator,
            guild,
            guild,
            guild.member(account_route["account"]),
        )
        return RelationshipDestroyed(context, active=False)

    @m.entity(QQAPICapability.event_callback, event_type="channel_create")
    async def channel_create(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        channel = guild.channel(str(raw_event["id"]))
        operator = guild.member(str(raw_event["op_user_id"]))
        context = Context(
            account,
            operator,
            channel.member(account_route["account"]),
            channel,
            channel.member(account_route["account"]),
        )
        return RelationshipCreated(context)

    @m.entity(QQAPICapability.event_callback, event_type="channel_delete")
    async def channel_delete(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        channel = guild.channel(str(raw_event["id"]))
        operator = guild.member(str(raw_event["op_user_id"]))
        context = Context(
            account,
            operator,
            channel,
            channel,
            channel.member(account_route["account"]),
        )
        return RelationshipDestroyed(context, active=False)

    @m.entity(QQAPICapability.event_callback, event_type="guild_member_add")
    async def guild_member_add(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        user = guild.member(str(raw_event["user"]["id"]))
        operator = guild.member(str(raw_event["op_user_id"]))
        context = Context(
            account,
            operator,
            user,
            guild,
            guild.member(account_route["account"]),
        )
        return RelationshipCreated(context)

    @m.entity(QQAPICapability.event_callback, event_type="guild_member_remove")
    async def guild_member_remove(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        user = guild.member(str(raw_event["user"]["id"]))
        operator = guild.member(str(raw_event["op_user_id"]))
        if raw_event["op_user_id"] != raw_event["user"]["id"]:
            context = Context(
                account,
                operator,
                user,
                guild,
                guild.member(account_route["account"]),
            )
            return RelationshipDestroyed(context, active=False)
        else:
            context = Context(
                account,
                user,
                guild,
                guild,
                guild.user(account_route["account"]),
            )
            return RelationshipDestroyed(context, active=True)

    @m.entity(QQAPICapability.event_callback, event_type="group_add_robot")
    async def group_add_robot(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        group = land.group(str(raw_event["group_openid"]))
        operator = group.member(str(raw_event["op_member_openid"]))
        context = Context(
            account,
            operator,
            group.member(account_route["account"]),
            group,
            group.member(account_route["account"]),
        )
        return RelationshipCreated(context)

    @m.entity(QQAPICapability.event_callback, event_type="group_del_robot")
    async def group_del_robot(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        group = land.group(str(raw_event["group_openid"]))
        operator = group.member(str(raw_event["op_member_openid"]))
        context = Context(
            account,
            operator,
            group.member(account_route["account"]),
            group,
            group.member(account_route["account"]),
        )
        return RelationshipDestroyed(context, active=False)

    @m.entity(QQAPICapability.event_callback, event_type="friend_add")
    async def friend_add(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account

        user = Selector().land("qq").friend(str(raw_event["openid"]))
        context = Context(
            account,
            user,
            account_route,
            user,
            account_route,
        )
        return RelationshipCreated(context)

    @m.entity(QQAPICapability.event_callback, event_type="friend_del")
    async def friend_del(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account

        user = Selector().land("qq").friend(str(raw_event["openid"]))
        context = Context(
            account,
            user,
            account_route,
            user,
            account_route,
        )
        return RelationshipDestroyed(context, active=False)



from __future__ import annotations

from datetime import timedelta

from graia.amnesia.builtins.memcache import Memcache, MemcacheService
from loguru import logger

from avilla.core.context import Context
from avilla.core.selector import Selector
from avilla.qqapi.capability import QQAPICapability
from avilla.qqapi.collector.connection import ConnectionCollector
from avilla.standard.core.activity import ActivityAvailable


class QQAPIEventActivityPerform((m := ConnectionCollector())._):
    m.namespace = "avilla.protocol/qqapi::event"
    m.identify = "activity"

    @m.entity(QQAPICapability.event_callback, event_type="interaction_create")
    async def button_interaction(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(self.connection.account_id)
        info = self.protocol.avilla.accounts.get(account_route)
        if info is None:
            logger.warning(f"Unknown account {self.connection.account_id} received message {raw_event}")
            return
        cache: Memcache = self.protocol.avilla.launch_manager.get_component(MemcacheService).cache
        account = info.account
        if raw_event["chat_type"] == 3:
            guild = Selector().land("qq").guild(raw_event["guild_id"])
            channel = guild.channel(raw_event["channel_id"])
            author = channel.member(raw_event["data"]["resolved"]["user_id"])
            context = Context(
                account,
                author,
                channel.member(account_route["account"]),
                channel,
                channel.member(account_route["account"]),
            )
            activity = channel.button(f'{raw_event["data"]["button_id"]}#{raw_event["data"]["button_data"]}')
            await cache.set(
                f"qqapi/account({account_route['account']}):{channel}", raw_event["id"], timedelta(minutes=5)
            )
        elif raw_event["chat_type"] == 2:
            friend = Selector().land("qq").friend(raw_event["data"]["resolved"]["user_id"])
            context = Context(account, friend, account_route, friend, account_route)
            activity = friend.button(f'{raw_event["data"]["button_id"]}#{raw_event["data"]["button_data"]}')
            await cache.set(
                f"qqapi/account({account_route['account']}):{friend}", raw_event["id"], timedelta(minutes=5)
            )
        else:
            group = Selector().land("qq").group(raw_event["group_open_id"])
            member = group.member(raw_event["data"]["resolved"]["user_id"])
            context = Context(
                account,
                member,
                group,
                group,
                group.member(account_route["account"]),
            )
            activity = group.button(f'{raw_event["data"]["button_id"]}#{raw_event["data"]["button_data"]}')
            await cache.set(f"qqapi/account({account_route['account']}):{group}", raw_event["id"], timedelta(minutes=5))
        return ActivityAvailable(context, "button_interaction", context.scene, activity)



from __future__ import annotations

from datetime import datetime

from avilla.core.context import Context
from avilla.core.selector import Selector
from avilla.qqapi.audit import Audit, MessageAuditPass, MessageAuditReject
from avilla.qqapi.capability import QQAPICapability
from avilla.qqapi.collector.connection import ConnectionCollector


class QQAPIEventAuditPerform((m := ConnectionCollector())._):
    m.namespace = "avilla.protocol/qqapi::event"
    m.identify = "audit"

    @m.entity(QQAPICapability.event_callback, event_type="message_audit_pass")
    async def on_message_audit_pass(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        channel = guild.channel(str(raw_event["channel_id"]))
        message = channel.message(str(raw_event["message_id"]))
        audit = Audit(
            raw_event["audit_id"],
            channel,
            raw_event["seq_in_channel"],
            datetime.fromisoformat(raw_event["audit_time"]),
            datetime.fromisoformat(raw_event["audit_time"]),
            message,
        )
        context = Context(
            account,
            channel,
            channel.member(account_route["account"]),
            channel,
            channel.member(account_route["account"]),
        )
        return MessageAuditPass(context, audit)

    @m.entity(QQAPICapability.event_callback, event_type="message_audit_reject")
    async def on_message_audit_reject(self, event_type: ..., raw_event: dict):
        account_route = Selector().land("qq").account(str(self.connection.account_id))
        account = self.protocol.avilla.accounts[account_route].account
        land = Selector().land("qq")
        guild = land.guild(str(raw_event["guild_id"]))
        channel = guild.channel(str(raw_event["channel_id"]))
        audit = Audit(
            raw_event["audit_id"],
            channel,
            raw_event["seq_in_channel"],
            datetime.fromisoformat(raw_event["audit_time"]),
            datetime.fromisoformat(raw_event["audit_time"]),
        )
        context = Context(
            account,
            channel,
            channel.member(account_route["account"]),
            channel,
            channel.member(account_route["account"]),
        )
        return MessageAuditReject(context, audit)



from collections import defaultdict

from avilla.core.platform import Abstract, Land, Platform

PRIVILEGE_TRANS = defaultdict(
    lambda: "read",
    {
        1: "read",
        2: "manage",
        3: "read+manage",
        4: "speak",
        5: "read+speak",
        6: "manage+speak",
        7: "read+manage+speak",
        8: "live",
        9: "read+live",
        10: "manage+live",
        11: "read+manage+live",
        12: "speak+live",
        13: "read+speak+live",
        14: "manage+speak+live",
        15: "read+manage+speak+live",
    },
)
PLATFORM = Platform(
    Land(
        "qq",
        [{"name": "Tencent"}],
        humanized_name="QQ API",
    ),
    Abstract(
        "qq-api-official",
        [{"name": "Tencent"}],
        humanized_name="QQ API",
    ),
    Abstract(
        "qqapi",
        [{"name": "GraiaProject"}],
        humanized_name="QQ/QQ-Guild Protocol for Avilla, use offical API",
    ),
)



from __future__ import annotations

from typing import TYPE_CHECKING

from avilla.core.ryanvk.collector.account import AccountCollector
from avilla.core.selector import Selector
from avilla.standard.core.profile import Summary, Nick

if TYPE_CHECKING:
    from avilla.qqapi.account import QQAPIAccount  # noqa
    from avilla.qqapi.protocol import QQAPIProtocol  # noqa


class QQAPIUserActionPerform((m := AccountCollector["QQAPIProtocol", "QQAPIAccount"]())._):
    m.namespace = "avilla.protocol/qqapi::action"
    m.identify = "user"

    @m.pull("land.guild.user", Summary)
    async def get_summary(self, target: Selector, route: ...) -> Summary:
        result = await self.account.connection.call_http("get", f"guilds/{target.pattern['guild']}", {})
        return Summary(result["name"], result["description"])

    @m.pull("land.guild.user", Nick)
    async def get_nick(self, target: Selector, route: ...) -> Nick:
        result = await self.account.connection.call_http("get", f"guilds/{target.pattern['guild']}", {})
        return Nick(result["name"], result["name"], None)



from .capability import RoleCreate as RoleCreate
from .capability import RoleDelete as RoleDelete
from .capability import RoleEdit as RoleEdit
from .capability import RoleMemberCapability as RoleMemberCapability
from .metadata import Role as Role



from __future__ import annotations

import asyncio
from contextlib import suppress
from datetime import datetime
from typing import TYPE_CHECKING, AsyncIterator, Literal

from loguru import logger
from typing_extensions import Self

from avilla.core.ryanvk.staff import Staff
from avilla.qqapi.audit import MessageAudited, audit_result
from avilla.qqapi.capability import QQAPICapability

from .util import Opcode, Payload

if TYPE_CHECKING:
    from avilla.qqapi.protocol import QQAPIProtocol

CallMethod = Literal["get", "post", "fetch", "update", "multipart", "put", "delete", "patch"]


class QQAPINetworking:
    protocol: QQAPIProtocol
    response_waiters: dict[str, asyncio.Future]
    close_signal: asyncio.Event

    account_id: str
    self_info: dict
    sequence: int | None
    session_id: str | None
    _access_token: str | None
    _expires_in: datetime | None

    def __init__(self, protocol: QQAPIProtocol):
        super().__init__()
        self.protocol = protocol
        self.response_waiters = {}
        self.close_signal = asyncio.Event()
        self.session_id = None
        self.sequence = None
        self._access_token = None
        self._expires_in = None

    def get_staff_components(self):
        return {"connection": self, "protocol": self.protocol, "avilla": self.protocol.avilla}

    def get_staff_artifacts(self):
        return [self.protocol.artifacts, self.protocol.avilla.global_artifacts]

    @property
    def staff(self):
        return Staff(self.get_staff_artifacts(), self.get_staff_components())

    def message_receive(self, shard: tuple[int, int]) -> AsyncIterator[tuple[Self, dict]]:
        ...

    @property
    def alive(self) -> bool:
        ...

    async def wait_for_available(self):
        ...

    async def send(self, payload: dict, shard: tuple[int, int]) -> None:
        ...

    async def message_handle(self, shard: tuple[int, int]):
        async for connection, data in self.message_receive(shard):
            if data["op"] != Opcode.DISPATCH:
                logger.debug(f"received other payload: {data}")
                continue
            payload = Payload(**data)
            connection.sequence = payload.sequence  # type: ignore

            async def event_parse_task(_data: Payload):
                event_type = _data.type
                if not event_type:
                    raise ValueError("event type is None")
                with suppress(NotImplementedError):
                    event = await QQAPICapability(connection.staff).event_callback(event_type.lower(), _data.data)
                    if event is not None:
                        if isinstance(event, MessageAudited):
                            audit_result.add_result(event)
                        await self.protocol.post_event(event)  # type: ignore
                    return
                logger.warning(f"received unsupported event {event_type.lower()}: {_data.data}")
                return

            asyncio.create_task(event_parse_task(payload))

    async def connection_closed(self):
        self.session_id = None
        self.close_signal.set()

    async def call_http(self, method: CallMethod, action: str, params: dict | None = None) -> dict:
        ...
